package com.shixing.sxedit;

import com.shixing.sxedit.util.PointF;
import com.shixing.sxedit.util.Size;
import com.shixing.sxedit.util.TextUtils;

/**
 * 合成
 * */
public class SXComposite {

    /**
     * 获取合成 Id
     *
     * */
    public String getCompositeId() {
        return mCompositeId;
    }

    /**
     * 获取合成尺寸
     */
    public Size  getSize() {
        if (mNativeManager > 0) {
            int[] size = nCompositeSize(mNativeManager, mCompositeId);
            if (size != null) {
                return new Size(size[0], size[1]);
            }
        }
        return null;
    }
    /**
     * 合成帧率
     *
     * @return 帧率
     */
    public double  getFps() {
        if (mNativeManager > 0) {
            return nFps(mNativeManager, mCompositeId);
        }
        return 0;
    }
    /**
     * 合成时长
     * @return 时长单位为秒
     */
    public double getDuration() {
        if (mNativeManager > 0) {
            return nDuration(mNativeManager, mCompositeId);
        }
        return 0;
    }
    /**
     * 合成帧数
     * @return 帧数
     */
    public long getFrames() {
        if (mNativeManager > 0) {
            return nFrames(mNativeManager, mCompositeId);
        }
        return 0;
    }
    /**
     * 获取合成中轨道的最大结束时间
     *
     * @param excludeAudio 是否排除音频时长
     * @return 轨道的最大结束时间
     */
    public double getContentMaxTime(boolean excludeAudio) {
        if (mNativeManager > 0) {
            return nContentMaxTime(mNativeManager, mCompositeId, excludeAudio);
        }
        return 0;
    }
    /**
     * 通过id获取对应的组
     * @param groupId 组的uuid
     * @return 组的实例对象，不存在对应的组时返回nullptr
     */
    public SXTrackGroup getGroup(String groupId) {
        if (mNativeManager > 0) {
            long group =  nGroup(mNativeManager, mCompositeId, groupId);
            long composite = nComposite(mNativeManager, mCompositeId);
            if (group > 0 && composite > 0) {
                return new SXTrackGroup(group, composite, SXTrackGroup.SXGroupOwnType.Composite);
            }
        }
        return null;
    }
    /**
     * 组是顺序存储的，在末尾追加一个组
     *
     * @return 新组的实例对象
     */
    public SXTrackGroup addNewGroup() {
        if (mNativeManager > 0) {
            long group =  nAddNewGroup(mNativeManager, mCompositeId);
            long composite = nComposite(mNativeManager, mCompositeId);
            if (group > 0 && composite > 0) {
                return new SXTrackGroup(group, composite, SXTrackGroup.SXGroupOwnType.Composite);
            }
        }
        return null;
    }
    /**
     * 组是顺序存储的，因此可以在index所在的位置插入一个组
     *
     * @param index 位置的索引，index超出数组个数或小于0时添加到末尾
     * @return 新组的实例对象
     */
    public SXTrackGroup addNewGroupAt(int index) {
        if (mNativeManager > 0) {
            long group = nAddGroupAt(mNativeManager, mCompositeId, index);
            long composite = nComposite(mNativeManager, mCompositeId);
            if (group > 0 && composite > 0) {
                return new SXTrackGroup(group, composite, SXTrackGroup.SXGroupOwnType.Composite);
            }
        }
        return null;
    }
    /**
     * 通过id删除组，如果组内存在轨道则会移除所有轨道，但不会删除轨道
     *
     * @param groupId 组 id
     */
    public void deleteGroup(String groupId) {
        if (mNativeManager > 0) {
            nDeleteGroup(mNativeManager, mCompositeId, groupId);
        }
    }
    /**
     * 获取所有的组，不包含主轨道
     * @return 顺序存储的所有组
     */
    public SXTrackGroup[] groups() {
        if (mNativeManager > 0) {
            long[] nativeGroups = nGroups(mNativeManager, mCompositeId);
            if (nativeGroups != null) {
                SXTrackGroup[] groups = new SXTrackGroup[nativeGroups.length];
                for (int i = 0; i < nativeGroups.length; i++) {
                    groups[i] = new SXTrackGroup(nativeGroups[i], mNativeManager, SXTrackGroup.SXGroupOwnType.Composite);
                }
                return groups;
            }
        }
        return null;
    }
    /**
     * 移动一个组到指定的位置
     *
     * @param groupId 组的id
     * @param index 组新的位置
     * @return 组查找失败或索引越界返回false
     */
    public void moveGroupTo(String groupId, int index) {
        if (mNativeManager > 0) {
            nMoveGroupTo(mNativeManager, mCompositeId, groupId, index);
        }
    }
    /**
     * 创建媒体轨道，接受视频和图片素材
     * <p>
     * 图片轨道时长默认时长为3秒，其他素材使用素材自身时长
     * </p>
     *
     * @param sourcePath 资源路径
     * @return 失败返回null
     */
    public SXMediaTrack createMediaTrack(String sourcePath) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(sourcePath)) {
            long track = nCreateMediaTrack(mNativeManager, mCompositeId, sourcePath);
            long composite = nComposite(mNativeManager, mCompositeId);
            if (track > 0) {
                return new SXMediaTrack(track, composite, SXTrack.SXTrackOwnType.Composite);
            }
        }
        return null;
    }
    /**
     * 创建贴纸轨道
     * <p>
     * 贴纸轨道时长默认使用贴纸包自身时长
     * </p>
     *
     * @param sourcePath 资源
     * @param duration 时长, 单位: 秒
     * @return 失败返回nullptr
     */
    public SXStickerTrack createStickerTrack(String sourcePath, double duration) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(sourcePath)) {
            long track = nCreateStickerTrack(mNativeManager, mCompositeId, sourcePath, duration);
            if (track > 0) {
                return new SXStickerTrack(track, mNativeManager, SXTrack.SXTrackOwnType.Composite);
            }
        }
        return null;
    }
    /**
     * 创建文字轨道
     *
     * @param duration  持续时长，文字轨道必须指定时长, 单位: 秒
     * @return 失败返回nullptr
     * */
    public SXTextTrack createTextTrack(double duration) {
        if (mNativeManager > 0) {
            long textTrack = nCreateTextTrack(mNativeManager, mCompositeId, duration);
            if (textTrack > 0) {
                return new SXTextTrack(textTrack, mNativeManager, SXTrack.SXTrackOwnType.Composite);
            }
        }
        return null;
    }

    /**
     * 创建音频轨道
     * @note 音频轨道时长默认使用音频自身时长
     * @param sourcePath 音频资源路径
     * @param duration 时长, 单位: 秒, 传 0 表示使用资源自身的时长
     * @return 失败返回nullptr
     */
    public SXAudioTrack createAudioTrack(String sourcePath, double duration) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(sourcePath)) {
            long track = nCreateAudioTrack(mNativeManager,mCompositeId, sourcePath, duration);
            if (track > 0) {
                return new SXAudioTrack(track, mNativeManager, SXTrack.SXTrackOwnType.Composite);
            }
        }
        return null;
    }

    /**
     * 通过轨道id删除一个轨道
     *
     * @param trackId 轨道id
     * @note 建议在删除前先从组中移除此轨道
     */
    public void deleteTrack(String trackId) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(trackId)) {
            nDeleteTrack(mNativeManager, mCompositeId, trackId);
        }
    }

    /**
     * 通过轨道id复制一个新的轨道
     *
     * @param trackId 轨道id
     * @return	新的轨道，复制失败时返回 null
     */
    public SXTrack cloneTrack(String trackId) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(trackId)) {
            long track = nCloneTrack(mNativeManager, mCompositeId, trackId);
            if (track > 0) {
                return SXTrack.createNewTrack(track, nComposite(mNativeManager, mCompositeId), SXTrack.SXTrackOwnType.Composite);
            }
        }
        return null;
    }

    /**
     * 通过轨道id 获取轨道
     *
     * @param trackId 轨道id
     * @return	轨道，失败时返回 null
     */
    public SXTrack getTrack(String trackId) {
        if (mNativeManager > 0 && !TextUtils.isEmpty(trackId)) {
            long track = nTrack(mNativeManager, mCompositeId, trackId);
            if (track > 0) {
                return SXTrack.createNewTrack(track, mNativeManager, SXTrack.SXTrackOwnType.Composite);
            }
        }
        return null;
    }

    public SXTrack hitTest(PointF point) {
        if (mNativeManager > 0) {
            long track = nHitTest(mNativeManager, mCompositeId, point.x, point.y);
            if (track > 0) {
                return SXTrack.createNewTrack(track, mNativeManager, SXTrack.SXTrackOwnType.Composite);
            }
        }
        return null;
    }

    public SXTrack hitTest(PointF point, int type) {
        if (mNativeManager > 0) {
            long track = nHitTest(mNativeManager, mCompositeId,  point.x, point.y, type);
            if (track > 0) {
                return SXTrack.createNewTrack(track, mNativeManager, SXTrack.SXTrackOwnType.Composite);
            }
        }
        return null;
    }


    private String mCompositeId;
    private long mNativeManager = 0;

    SXComposite(long composite, long manager) {
        mCompositeId = nCompositeId(composite);
        mNativeManager = manager;
    }

    public long getNativeEditManager() {
        return mNativeManager;
    }

    static native long nEditManager(long nativeComposite);
    static native long nComposite(long nativeManager, String mCompositeId);
    static native String nCompositeId(long nativeComposite);
    static native int[] nCompositeSize(long nativeManager, String compositeId);
    static native double nFps(long nativeManager, String compositeId);
    static native double nDuration(long nativeManager, String compositeId);
    static native long nFrames(long nativeManager, String compositeId);
    static native double nContentMaxTime(long nativeManager, String compositeId, boolean excludeAudio);

    static native long nGroup(long nativeManager, String compositeId, String groupId);
    static native long nAddNewGroup(long nativeManager, String compositeId);
    static native long nAddGroupAt(long nativeManager, String compositeId, int index);
    static native void nDeleteGroup(long nativeManager, String compositeId, String groupId);
    static native void nMoveGroupTo(long nativeManager, String compositeId, String groupId, int index);
    static native long[] nGroups(long nativeManager, String compositeId);

    static native long nCreateMediaTrack(long nativeManager, String compositeId, String sourcePath);
    static native long nCreateStickerTrack(long nativeManager, String compositeId, String sourcePath, double duration);
    static native long nCreateTextTrack(long nativeManager, String compositeId, double duration);
    static native long nCreateAudioTrack(long nativeManager, String compositeId, String sourcePath, double duration);
    static native void nDeleteTrack(long nativeManager, String compositeId, String trackId);
    static native long nTrack(long nativeManager, String compositeId, String trackId);
    static native long nCloneTrack(long nativeManager, String compositeId, String trackId);

    static native long nHitTest(long nativeManager, String compositeId, float x, float y);
    static native long nHitTest(long nativeManager, String compositeId, float x, float y, int type);
}
